@@ -75,11 +75,15 @@ class Renamer(object):
             move_to = move_to.replace(ref_string, replacement)
         return TRANSFORM[self.xform](move_to)
 
-    def _process_single(self, entry, match, target, index, max_indexes):
-        if not match:
-            return
-        move_to = self._apply_match(match, target, index, max_indexes)
-        self.targets.setdefault(move_to, []).append(entry)
+    def _apply_replace(self, match, substring_from, substring_to, index,
+        max_indexes):
+        substring_from = re.escape(substring_from)
+        if not self.case_insensitive:
+            sub = re.compile(substring_from)
+        else:
+            sub = re.compile(substring_from, re.IGNORECASE)
+        move_to = sub.sub(substring_to, match.group())
+        return TRANSFORM[self.xform](move_to)
 
     def index(self, number):
         return self.index_first + (number * self.index_step)
@@ -88,9 +92,10 @@ class Renamer(object):
         return (os.path.abspath(file1).lower() == os.path.abspath(file2).lower()
             and os.stat(file1).st_ino == os.stat(file2).st_ino)
 
-    def rename(self, regex, target, except_regex=None, **kwargs):
+    def _rename(self, regex, target=None, except_regex=None,
+        substring_from=None, substring_to=None, **kwargs):
         if self.quiet:
-            INFO, ERROR = open(os.devnull), open(os.devnull)
+            INFO = ERROR = open(os.devnull, 'w')
         else:
             INFO, ERROR = sys.stdout, sys.stderr
         flags = 0
@@ -107,7 +112,16 @@ class Renamer(object):
         file_count = len(files)
         for index, entry in enumerate(files):
             m = r.search(entry)
-            self._process_single(entry, m, target, index, file_count)
+            if not m:
+                continue
+            if target is not None:
+                # classic mode
+                move_to = self._apply_match(m, target, index, file_count)
+            else:
+                # simple mode
+                move_to = self._apply_replace(m, substring_from, substring_to,
+                    index, file_count)
+            self.targets.setdefault(move_to, []).append(entry)
         # sanity check
         for target, sources in self.targets.iteritems():
             if len(sources) > 1:
@@ -122,6 +136,11 @@ class Renamer(object):
                 return 2
         # actual rename
         for target, sources in self.targets.iteritems():
+            if sources[0] == target:
+                if self.test:
+                    print >>INFO, ("File %s matches but name doesn't change."
+                        % (sources[0],))
+                continue
             if self.copy:
                 if self.test:
                     print >>INFO, ("Would run shutil.copy(%s, %s) (with "
@@ -138,27 +157,66 @@ class Renamer(object):
                     os.rename(sources[0], target)
         return 0
 
+    def rename(self, regex, target, except_regex=None, **kwargs):
+        return self._rename(regex, target, except_regex=except_regex,
+            **kwargs)
+
+    def rename_simple(self, substring_from, substring_to, regex,
+        except_regex=None, **kwargs):
+        return self._rename(regex, substring_from=substring_from,
+            substring_to=substring_to, except_regex=except_regex, **kwargs)
+
+
+class ProxyMember(object):
+    def __init__(self, name, targets):
+        self.name = name
+        self.targets = targets
+
+    def __call__(self, *args, **kwargs):
+        result = []
+        for target in self.targets:
+            result.append(getattr(target, self.name)(*args, **kwargs))
+        return result
+
+
+class Proxy(object):
+    def __init__(self, *targets):
+        self.targets = targets
+
+    def __getattr__(self, name):
+        return ProxyMember(name, self.targets)
+
 
 def run():
-    import argparse
-    parser = argparse.ArgumentParser(prog='rename')
-    parser.add_argument('-c', '--copy', action='store_true',
+    try:
+        import argparse
+    except ImportError:
+        print sys.stderr, "rename requires the `argparse` library."
+        sys.exit(100)
+    stparser = argparse.ArgumentParser(prog='rename', add_help=False)
+    classic = argparse.ArgumentParser(prog='rename')
+    simple = argparse.ArgumentParser(prog='rename')
+    invocator = Proxy(classic, stparser)
+    simple.add_argument('-s', '--simple', action='store_true', help='invokes '
+        'the simple mode', required=True)
+    common = Proxy(classic, simple)
+    common.add_argument('-c', '--copy', action='store_true',
         help='copy files instead of renaming')
-    parser.add_argument('-I', '--case-insensitive', action='store_true',
+    common.add_argument('-I', '--case-insensitive', action='store_true',
         help='treat the regular expression as case-insensitive')
-    parser.add_argument('-l', '--lower', action='store_const', dest='xform',
+    common.add_argument('-l', '--lower', action='store_const', dest='xform',
         const='lower', help='translate all letters to lower-case')
-    parser.add_argument('-q', '--quiet', action='store_true',
+    common.add_argument('-q', '--quiet', action='store_true',
         help='don\'t print anything, just return status codes')
-    parser.add_argument('-U', '--upper', action='store_const', dest='xform',
+    common.add_argument('-U', '--upper', action='store_const', dest='xform',
         const='upper', help='translate all letters to upper-case')
-    parser.add_argument('-v', '--except', dest='except_regex', action='store',
+    common.add_argument('-v', '--except', dest='except_regex', action='store',
         default="", help='exclude files matching the following '
         'regular expression')
-    parser.add_argument('-t', '--test', action='store_true', help='test only, '
+    common.add_argument('-t', '--test', action='store_true', help='test only, '
         'don\'t actually rename anything')
-    group = parser.add_argument_group('Configuration for the special \\(index)'
-        ' reference')
+    group = classic.add_argument_group('Configuration for the special '
+        '\\(index) reference')
     group.add_argument('--index-first', default=1, help='specifies '
         'what number will the first \\(index) substitution contain. Default: 1')
     group.add_argument('--index-step', default=1, help='specifies '
@@ -172,24 +230,35 @@ def run():
         ' of characters)')
     group.add_argument('--index-pad-with', default='0',
         help='specifies what character will be used for padding. Default: "0"')
-    parser.add_argument('--selftest', nargs='?', const=True,
+    invocator.add_argument('-s', '--simple', action='store_true', help='invokes '
+        'the simple mode. For more help on its positional arguments: '
+        'rename -s --help')
+    invocator.add_argument('--selftest', nargs='?', const=True,
         metavar='use_directory', help='run internal unit tests')
-    parser.add_argument('regex', help='regular expression to match '
+    classic.add_argument('regex', help='regular expression to match '
         'files with')
-    parser.add_argument('target', help='target pattern using '
+    classic.add_argument('target', help='target pattern using '
         'references to groups in the regular expression')
-    stparser = argparse.ArgumentParser(prog='rename', add_help=False)
-    stparser.add_argument('--selftest', nargs='?', const=True,
-        help='run internal unit tests')
+    simple.add_argument('substring_from', help='simple (raw) substring that '
+        'should be found within the filename')
+    simple.add_argument('substring_to', help='the replacement string')
+    simple.add_argument('regex', help='regular expression to match '
+        'files with')
     args = stparser.parse_known_args()
-    if not args[0].selftest:
-        args = parser.parse_args()
+    if args[0].selftest:
+        selftest(args[0].selftest)
+    elif args[0].simple:
+        args = simple.parse_args()
+        args.regex = "".join(args.regex)
+        args.substring_from = "".join(args.substring_from)
+        args.substring_to = "".join(args.substring_to)
+        sys.exit(Renamer(**args.__dict__).rename_simple(**args.__dict__))
+    else:
+        args = classic.parse_args()
         args.regex = "".join(args.regex)
         args.except_regex = "".join(args.except_regex)
         args.target = "".join(args.target)
         sys.exit(Renamer(**args.__dict__).rename(**args.__dict__))
-    else:
-        selftest(args[0].selftest)
 
 def selftest(temp_dir=None):
     if temp_dir == True:
@@ -249,7 +318,10 @@ def selftest(temp_dir=None):
             os.chdir(dirpath)
             try:
                 try:
-                    result = Renamer(**kwargs).rename(**kwargs)
+                    if not kwargs.get('simple'):
+                        result = Renamer(**kwargs).rename(**kwargs)
+                    else:
+                        result = Renamer(**kwargs).rename_simple(**kwargs)
                     if result != kwargs.get('result', 0):
                         raise ValueError(result)
                     files = set(os.listdir('.'))
@@ -259,11 +331,14 @@ def selftest(temp_dir=None):
                     os.chdir(cwd)
                 print "Test %(name)s OK." % kwargs
                 return 0
-            except ValueError:
-                print >>sys.stderr, "Test %(name)s (%(desc)s) failed." % kwargs
+            except ValueError, e:
+                kwargs['result'] = e
+                print >>sys.stderr, "Test %(name)s (%(desc)s) failed "\
+                    "(%(result)s)." % kwargs
                 return result
             except:
-                print >>sys.stderr, "Test %(name)s (%(desc)s) failed." % kwargs
+                print >>sys.stderr, "Test %(name)s (%(desc)s) failed (12)."\
+                    "" % kwargs
                 return 12
         return _runtest(case)
 
@@ -336,6 +411,55 @@ def selftest(temp_dir=None):
                 regex=r'CaSe(\d[qwertyuiop])', target=r'C\(index)',
                 index_first=100, index_step=2, index_pad_with='_',
                 files=['C{0:_>3}'.format(i) for i in xrange(100, 220, 2)])
+        i += _runcase(name="10", desc='CaSe -> replace `e` with `ee`',
+                simple=True, regex=r'CaSe(\d[qwertyuiop])',
+                substring_from='e', substring_to='ee',
+                files=['CaSee1ee', 'CaSee1i', 'CaSee1o', 'CaSee1p', 'CaSee1q',
+                       'CaSee1r', 'CaSee1t', 'CaSee1u', 'CaSee1w', 'CaSee1y',
+                       'CaSee2ee', 'CaSee2i', 'CaSee2o', 'CaSee2p', 'CaSee2q',
+                       'CaSee2r', 'CaSee2t', 'CaSee2u', 'CaSee2w', 'CaSee2y',
+                       'CaSee3ee', 'CaSee3i', 'CaSee3o', 'CaSee3p', 'CaSee3q',
+                       'CaSee3r', 'CaSee3t', 'CaSee3u', 'CaSee3w', 'CaSee3y',
+                       'case1e', 'case1i', 'case1o', 'case1p', 'case1q',
+                       'case1r', 'case1t', 'case1u', 'case1w', 'case1y',
+                       'case2e', 'case2i', 'case2o', 'case2p', 'case2q',
+                       'case2r', 'case2t', 'case2u', 'case2w', 'case2y',
+                       'case3e', 'case3i', 'case3o', 'case3p', 'case3q',
+                       'case3r', 'case3t', 'case3u', 'case3w', 'case3y'])
+        i += _runcase(name="11", desc='CaSe (i) -> replace `e` with `ee`',
+                simple=True, case_insensitive=True, regex=r'CaSe(\d[qwertyuiop])',
+                substring_from='e', substring_to='ee',
+                files=['CaSee1ee', 'CaSee1i', 'CaSee1o', 'CaSee1p', 'CaSee1q',
+                       'CaSee1r', 'CaSee1t', 'CaSee1u', 'CaSee1w', 'CaSee1y',
+                       'CaSee2ee', 'CaSee2i', 'CaSee2o', 'CaSee2p', 'CaSee2q',
+                       'CaSee2r', 'CaSee2t', 'CaSee2u', 'CaSee2w', 'CaSee2y',
+                       'CaSee3ee', 'CaSee3i', 'CaSee3o', 'CaSee3p', 'CaSee3q',
+                       'CaSee3r', 'CaSee3t', 'CaSee3u', 'CaSee3w', 'CaSee3y',
+                       'casee1ee', 'casee1i', 'casee1o', 'casee1p', 'casee1q',
+                       'casee1r', 'casee1t', 'casee1u', 'casee1w', 'casee1y',
+                       'casee2ee', 'casee2i', 'casee2o', 'casee2p', 'casee2q',
+                       'casee2r', 'casee2t', 'casee2u', 'casee2w', 'casee2y',
+                       'casee3ee', 'casee3i', 'casee3o', 'casee3p', 'casee3q',
+                       'casee3r', 'casee3t', 'casee3u', 'casee3w', 'casee3y'])
+        i += _runcase(name="12", desc='CaSe (i) -> (U) replace `e` with `ee`',
+                simple=True, case_insensitive=True, regex=r'CaSe(\d[qwertyuiop])',
+                substring_from='e', substring_to='ee', xform='upper', result=1)
+        i += _runcase(name="13", desc='CaSe (i) -> replace `e` with `ee` '\
+                '(except e$)',
+                simple=True, case_insensitive=True, regex=r'CaSe(\d[qwertyuiop])',
+                substring_from='e', substring_to='ee', except_regex=r'e$',
+                files=['CaSe1e', 'CaSee1i', 'CaSee1o', 'CaSee1p', 'CaSee1q',
+                       'CaSee1r', 'CaSee1t', 'CaSee1u', 'CaSee1w', 'CaSee1y',
+                       'CaSe2e', 'CaSee2i', 'CaSee2o', 'CaSee2p', 'CaSee2q',
+                       'CaSee2r', 'CaSee2t', 'CaSee2u', 'CaSee2w', 'CaSee2y',
+                       'CaSe3e', 'CaSee3i', 'CaSee3o', 'CaSee3p', 'CaSee3q',
+                       'CaSee3r', 'CaSee3t', 'CaSee3u', 'CaSee3w', 'CaSee3y',
+                       'case1e', 'casee1i', 'casee1o', 'casee1p', 'casee1q',
+                       'casee1r', 'casee1t', 'casee1u', 'casee1w', 'casee1y',
+                       'case2e', 'casee2i', 'casee2o', 'casee2p', 'casee2q',
+                       'casee2r', 'casee2t', 'casee2u', 'casee2w', 'casee2y',
+                       'case3e', 'casee3i', 'casee3o', 'casee3p', 'casee3q',
+                       'casee3r', 'casee3t', 'casee3u', 'casee3w', 'casee3y'])
         return i
 
     def _case_preserving_tests():
@@ -409,6 +533,44 @@ def selftest(temp_dir=None):
                 regex=r'CaSe(\d[qwertyuiop])', target=r'C\(index)',
                 index_first=100, index_step=2, index_pad_with='_',
                 files=['C{0:_>3}'.format(i) for i in xrange(100, 160, 2)])
+        i += _runcase(name="10", desc='CaSe -> replace `e` with `ee`',
+                simple=True, regex=r'CaSe(\d[qwertyuiop])',
+                substring_from='e', substring_to='ee',
+                files=['CaSee1ee', 'CaSee1i', 'CaSee1o', 'CaSee1p', 'CaSee1q',
+                       'CaSee1r', 'CaSee1t', 'CaSee1u', 'CaSee1w', 'CaSee1y',
+                       'CaSee2ee', 'CaSee2i', 'CaSee2o', 'CaSee2p', 'CaSee2q',
+                       'CaSee2r', 'CaSee2t', 'CaSee2u', 'CaSee2w', 'CaSee2y',
+                       'CaSee3ee', 'CaSee3i', 'CaSee3o', 'CaSee3p', 'CaSee3q',
+                       'CaSee3r', 'CaSee3t', 'CaSee3u', 'CaSee3w', 'CaSee3y'])
+        i += _runcase(name="11", desc='CaSe -> replace `e` with `ee` '\
+                '(except e$)',
+                simple=True, regex=r'CaSe(\d[qwertyuiop])',
+                substring_from='e', substring_to='ee', except_regex=r'e$',
+                files=['CaSe1e', 'CaSee1i', 'CaSee1o', 'CaSee1p', 'CaSee1q',
+                       'CaSee1r', 'CaSee1t', 'CaSee1u', 'CaSee1w', 'CaSee1y',
+                       'CaSe2e', 'CaSee2i', 'CaSee2o', 'CaSee2p', 'CaSee2q',
+                       'CaSee2r', 'CaSee2t', 'CaSee2u', 'CaSee2w', 'CaSee2y',
+                       'CaSe3e', 'CaSee3i', 'CaSee3o', 'CaSee3p', 'CaSee3q',
+                       'CaSee3r', 'CaSee3t', 'CaSee3u', 'CaSee3w', 'CaSee3y'])
+        i += _runcase(name="12", desc='CaSe -> replace `cAs` with `Fac`',
+                simple=True, regex=r'CaSe(\d[qwertyuiop])',
+                substring_from='cAs', substring_to='Fac', except_regex=r'e$',
+                files=['CaSe1e', 'CaSe1i', 'CaSe1o', 'CaSe1p', 'CaSe1q',
+                       'CaSe1r', 'CaSe1t', 'CaSe1u', 'CaSe1w', 'CaSe1y',
+                       'CaSe2e', 'CaSe2i', 'CaSe2o', 'CaSe2p', 'CaSe2q',
+                       'CaSe2r', 'CaSe2t', 'CaSe2u', 'CaSe2w', 'CaSe2y',
+                       'CaSe3e', 'CaSe3i', 'CaSe3o', 'CaSe3p', 'CaSe3q',
+                       'CaSe3r', 'CaSe3t', 'CaSe3u', 'CaSe3w', 'CaSe3y'])
+        i += _runcase(name="13", desc='CaSe -> replace `cAs` with `Fac`',
+                simple=True, regex=r'CaSe(\d[qwertyuiop])',
+                case_insensitive=True,
+                substring_from='cAs', substring_to='Fac', except_regex=r'e$',
+                files=['CaSe1e', 'Face1i', 'Face1o', 'Face1p', 'Face1q',
+                       'Face1r', 'Face1t', 'Face1u', 'Face1w', 'Face1y',
+                       'CaSe2e', 'Face2i', 'Face2o', 'Face2p', 'Face2q',
+                       'Face2r', 'Face2t', 'Face2u', 'Face2w', 'Face2y',
+                       'CaSe3e', 'Face3i', 'Face3o', 'Face3p', 'Face3q',
+                       'Face3r', 'Face3t', 'Face3u', 'Face3w', 'Face3y'])
         return i
 
     def _case_insensitive_tests():
